/*
** Module   : XNodeRef
** Abstract : DOM XML Node reference resource type interface.
**
** Copyright (c) 2013, Golden Code Development Corporation.
** ALL RIGHTS RESERVED. Use is subject to license terms.
**
**           Golden Code Development Corporation
**                      CONFIDENTIAL
**
** -#- -I- --Date-- -----------------------Description------------------------
** 001 EVL 20130130 Created initial version.
** 002 CA  20130221 Added LOCAL-NAME support. 
** 003 EVL 20130222 Adding NODE-VALUE attribute and SET-ATTRIBUTE-NODE method.
** 004 EVK 20130531 Adding NODE-VALUE attribute and GET-PARENT, GET-ATTRIBUTE-NODE, NORMALIZE
**                  LONGCHAR-TO-NODE-VALUE, NODE-VALUE-TO-LONGCHAR, CLONE-NODE methods.
*/

package com.goldencode.p2j.xml;

import com.goldencode.p2j.util.*;

/**
 * Interface describinf X-Node-Reference specific features of the DOM XML
 * objects.
 */
public interface XNodeRef
extends XEntity,
        LocalName
{
   // Attributes
   /**
    * Getting the attribute names available for current node reference.
    * 
    * @return   The comma separated character string of all available
    *           attributed.
    */
   public character getAttributeNames();

   /**
    * Getting the Parent of current node.
    *
    * @param    hNoderefParent
    *           A valid handle to assign parent XML node.
    *
    * @return   The <code>true</code> in case of success
    *           <code>false</code> otherwise.
    */
   public logical getParent(handle hNoderefParent);
   
   /**
    * Getting the value of the XML node value attribute.
    * 
    * @return   The character value of the node reference.
    */
   public character getNodeValue();
   
   /**
    * Setting the value of the XML node value attribute.
    * 
    * @param    value
    *           The new value of the node value attribute.
    */
   public void setNodeValue(character value);
   
   /**
    * Setting the value of the XML node value attribute.
    * 
    * @param    value
    *           The new value of the node value attribute.
    */
   public void setNodeValue(String value);
   
   // Methods
   /**
    * Deleting the internals of the current X-Node-Reference.
    * 
    * @return   The <code>true</code> in case of success
    *           <code>false</code> otherwise.
    */
   public logical deleteNode();
   
   /**
    * Getting the attribute value for a given attribute name.
    * 
    * @param    name
    *           The name of the attribute to return.
    *
    * @return   The comma separated character string of all available
    *           attributed.
    */
   public character getAttribute(String name);
   
   /**
    * Getting the attribute value for a given attribute name.
    * 
    * @param    name
    *           The name of the attribute to return.
    *
    * @return   The comma separated character string of all available
    *           attributed.
    */
   public character getAttribute(character name);
   
   /**
    * Setting the attribute value for a given attribute name.
    * 
    * @param    name
    *           The name of the attribute to change.
    * @param    value
    *           The new value for the given attribute name.
    * 
    * @return   The <code>true</code> in case of success
    *           <code>false</code> otherwise.
    */
   public logical setAttribute(String name, String value);

   /**
    * Setting the attribute value for a given attribute name.
    * 
    * @param    name
    *           The name of the attribute to change.
    * @param    value
    *           The new value for the given attribute name.
    * 
    * @return   The <code>true</code> in case of success
    *           <code>false</code> otherwise.
    */
   public logical setAttribute(String name, character value);

   /**
    * Setting the attribute value for a given attribute name.
    * 
    * @param    name
    *           The name of the attribute to change.
    * @param    value
    *           The new value for the given attribute name.
    * 
    * @return   The <code>true</code> in case of success
    *           <code>false</code> otherwise.
    */
   public logical setAttribute(character name, String value);

   /**
    * Setting the attribute value for a given attribute name.
    * 
    * @param    name
    *           The name of the attribute to change.
    * @param    value
    *           The new value for the given attribute name.
    * 
    * @return   The <code>true</code> in case of success
    *           <code>false</code> otherwise.
    */
   public logical setAttribute(character name, character value);

   /**
    * Associates XML attribute node with the given X-Noderef object handle.
    * 
    * @param    handle
    *           The handle that represents an XML attribute node created with create node or
    *           create node namespace.
    *
    * @return   The <code>true</code> in case of success <code>false</code> otherwise.
    */
   public logical setAttributeNode(handle handle);

   /**
    * Associates XML attribute node with the specified name to the given X-Noderef object
    * handle.
    *
    * @param    handle
    *           The handle that represents an XML attribute node with the specified name.
    * @param    attributeName
    *           The string that represents an XML attribute node with the specified name.
    *
    * @return   The <code>true</code> in case of success <code>false</code> otherwise.
    */
   public logical getAttributeNode(handle handle, String attributeName);

   /**
    * Associates XML attribute node with the specified name to the given X-Noderef object
    * handle.
    *
    * @param    handle
    *           The handle that represents an XML attribute node with the specified name.
    * @param    attributeName
    *           The character that represents an XML attribute node with the specified name.
    *
    * @return   The <code>true</code> in case of success <code>false</code> otherwise.
    */
   public logical getAttributeNode(handle handle, character attributeName);


   /**
    * Setting the binary value of the XML node value attribute.
    *
    * @param    value
    *           The new value of the node value attribute.
    *
    * @return   The <code>true</code> in case of success <code>false</code> otherwise.
    */
   public logical longcharToNodeValue(longchar value);

   /**
    * Setting the XML node value attribute to binary value of longchar.
    *
    * @param    value
    *           To save the value on XML node to this parameter.
    *
    * @return   The <code>true</code> in case of success <code>false</code> otherwise.
    */
   public logical nodeValueToLongchar(longchar value);

   /**
    * Setting the binary value of the XML node value attribute.
    *
    * @param    value
    *           The new value of the node value attribute.
    *
    * @return   The <code>true</code> in case of success <code>false</code> otherwise.
    */
   public logical memptrToNodeValue(memptr value);

   /**
    * Setting the XML node value attribute to binary value of memptr.
    *
    * @param    value
    *           To save the value on XML node to this parameter.
    *
    * @return   The <code>true</code> in case of success <code>false</code> otherwise.
    */
   public logical nodeValueToMemptr(memptr value);

   /**
    * Creating the new node within the current Document object.
    *
    * @param    xNodeRef
    *           The valid handle class instance to use for the cloned XML node.
    * @param    deep
    *           A logical that if <code>true</code> specifies that the whole sub-trees is to
    *           be copied. The default value is {@code false}.
    *
    * @return   The <code>true</code> in case of success <code>false</code> otherwise.
    */
   public logical cloneNode(handle xNodeRef, logical deep);


   /**
    * Creating the new node within the current Document object.
    *
    * @param    xNodeRef
    *           The valid handle class instance to use for the cloned XML node.
    * @param    deep
    *           A logical that if <code>true</code> specifies that the whole sub-trees is to
    *           be copied. The default value is {@code false}.
    *
    * @return   The <code>true</code> in case of success <code>false</code> otherwise.
    */
   public logical cloneNode(handle xNodeRef, boolean deep);

   /**
    * Normalizes TEXT and ATTRIBUTE nodes in the full depth of the sub-tree under this
    * XML node.
    *
    * @return   The <code>true</code> in case of success <code>false</code> otherwise.
    */
   public logical normalize();
}
