/*
** Module   : XmlFactory
** Abstract : DOM and SAX XML instance creation.
**
** Copyright (c) 2013, Golden Code Development Corporation.
** ALL RIGHTS RESERVED. Use is subject to license terms.
**
**           Golden Code Development Corporation
**                      CONFIDENTIAL
**
** -#- -I- --Date-- --------------------------------Description-----------------------------------
** 001 EVL 20130204 Created initial version. Major rework to use w3c backend.
** 002 EVL 20130208 Added SAX XML Support.
** 003 CA  20130308 The CREATE APIs need to receive a handle parameter, to properly initiate it,
**                  as returning the new handle instance and changing the handle reference in the
**                  caller is not the right approach.
** 004 CA  20130309 Added missing APIs related to WIDGET-POOL clause.
** 005 CA  20130311 Added more missing APIs related to WIDGET-POOL clause.
** 006 EVK 20130531 Added maps ABL node name to Dom nodeId and Dom nodeId to
**                  ABL node name. Added methods convertNodeToAblNode() and
**                  convertAblNodeToNodeId().
**                  createXNodeReference() methods now uses handle in parameters instead of
**                  Document interface.
**                  Added type support for createXNodeReference() method.
**
*/

package com.goldencode.p2j.xml;

import com.goldencode.p2j.util.character;
import com.goldencode.p2j.util.handle;
import org.w3c.dom.DOMException;
import org.w3c.dom.Document;
import org.w3c.dom.Node;

import java.util.HashMap;
import java.util.Map;

import static com.goldencode.p2j.xml.XmlTokenTypes.*;

/**
 * The set of methods to work with DOM XML and SAX support implementation in P2J.
 */
public class XmlFactory
{
   private static Map<String, Integer> nodeNameToNodeId = new HashMap<String, Integer>();
   private static Map<Integer, String> nodeIdToNodeName = new HashMap<Integer, String>();

   static
   {
      nodeNameToNodeId.put("ATTRIBUTE", ATTRIBUTE_NODE);
      nodeNameToNodeId.put("CDATA-SECTION", CDATA_SECTION_NODE);
      nodeNameToNodeId.put("COMMENT", COMMENT_NODE);
      nodeNameToNodeId.put("DOCUMENT", DOCUMENT_NODE);
      nodeNameToNodeId.put("DOCUMENT-FRAGMENT", DOCUMENT_FRAGMENT_NODE);
      nodeNameToNodeId.put("ELEMENT", ELEMENT_NODE);
      nodeNameToNodeId.put("ENTITY-REFERENCE", ENTITY_REFERENCE_NODE);
      nodeNameToNodeId.put("PROCESSING-INSTRUCTION", PROCESSING_INSTRUCTION_NODE);
      nodeNameToNodeId.put("TEXT", TEXT_NODE);
      for (Map.Entry<String, Integer> entry : nodeNameToNodeId.entrySet())
      {
         nodeIdToNodeName.put(entry.getValue(), entry.getKey());
      }
   }

   /**
    * Constructs a new instance of the Progress X-Document object and
    * retrieves the handle variable associated with it.
    * 
    * @param    h
    *           The handle where the new resource will be set as the referent.
    */
   public static void createXDocument(handle h)
   {
      h.assign(new XDocumentImpl());
   }

   /**
    * Constructs a new instance of the Progress X-Document object and
    * retrieves the handle variable associated with it.
    * 
    * @param    h
    *           The handle where the new resource will be set as the referent.
    * @param    poolName
    *           The name of the pool to which the resource will be added
    */
   public static void createXDocument(handle h, String poolName)
   {
      createXDocument(h, new character(poolName));
   }

   /**
    * Constructs a new instance of the Progress X-Document object and
    * retrieves the handle variable associated with it.
    * 
    * @param    h
    *           The handle where the new resource will be set as the referent.
    * @param    poolName
    *           The name of the pool to which the resource will be added
    */
   public static void createXDocument(handle h, character poolName)
   {
      h.assign(new XDocumentImpl());
      // TODO: add it to the pool
   }

   /**
    * Constructs a new instance of the Progress X-Node-Reference object and
    * retrieves the handle variable associated with it.
    * 
    * @param    h
    *           The handle where the new resource will be set as the referent.
    */
   public static void createXNodeReference(handle h)
   {
      h.assign(new XNodeRefImpl());
   }

   /**
    * Constructs a new instance of the Progress X-Node-Reference object and
    * retrieves the handle variable associated with it.
    * 
    * @param    h
    *           The handle where the new resource will be set as the referent.
    * @param    poolName
    *           The name of the pool to which the resource will be added
    */
   public static void createXNodeReference(handle h, String poolName)
   {
      createXNodeReference(h, new character(poolName));
   }

   /**
    * Constructs a new instance of the Progress X-Node-Reference object and
    * retrieves the handle variable associated with it.
    * 
    * @param    h
    *           The handle where the new resource will be set as the referent.
    * @param    poolName
    *           The name of the pool to which the resource will be added
    */
   public static void createXNodeReference(handle h, character poolName)
   {
      h.assign(new XNodeRefImpl());
      // TODO: add it to the pool
   }

   /**
    * Constructs a new instance of the Progress X-Node-Reference object and
    * retrieves the handle variable associated with it.
    *
    * @param    h
    *           The handle where the new resource will be set as the referent.
    * @param    docHandle
    *           handle should be valid and should contains the valid X-Document object to
    *           create X-Node under.
    * @param    name
    *           The name of the node to create.
    * @param    type
    *           The type of the node to create.
    * @throws   DOMException
    *           if type is cannot resolved. Or if name is invalid for XML.
    *
    *
    */
   static void createXNodeReference(handle h,
                                    handle docHandle,
                                    character name,
                                    character type)
   {
      if (docHandle == null || !docHandle._isValid())
      {
         throw new IllegalArgumentException("docHandle must be not null and valid");
      }

      Document doc = getDocument(docHandle);
      if (doc == null)
      {
         throw new IllegalArgumentException("docHandle must be of type DOCUMENT_NODE");
      }

      final Integer nodeId;
      if (type == null || type.isUnknown())
      {
         nodeId = ELEMENT_NODE;
      }
      else
      {
         nodeId = convertAblNodeToNodeId(type.toStringMessage());
      }

      if (nodeId == null)
      {
         throw new DOMException(DOMException.SYNTAX_ERR, "Unknown type");
      }

      String nameStr = null;
      if (!name.isUnknown())
      {
         nameStr = name.toStringMessage();
      }

      final Node node;

      switch (nodeId)
      {
         case ATTRIBUTE_NODE:
            node = doc.createAttribute(nameStr);
            break;
         case CDATA_SECTION_NODE:
            node = doc.createCDATASection("");
            break;
         case COMMENT_NODE:
            node = doc.createComment("");
            break;
         case DOCUMENT_FRAGMENT_NODE:
            node = doc.createDocumentFragment();
            break;
         case ELEMENT_NODE:
            node = doc.createElement(nameStr);
            break;
         case ENTITY_REFERENCE_NODE:
            node = doc.createEntityReference(nameStr);
            break;
         case PROCESSING_INSTRUCTION_NODE:
            node = doc.createProcessingInstruction(nameStr, "");
            break;
         case TEXT_NODE:
            node = doc.createTextNode("");
            break;
         default:
            throw new DOMException(DOMException.TYPE_MISMATCH_ERR, "Unknown type");
      }

      h.assign(new XNodeRefImpl(node));
   }
   /**
    * Extract {@link Document} from valid handle, which contains {@link XDocumentImpl} class.
    * If handle is not valid or doesn't inherit {@link XDocumentImpl} than return null.
    *
    * @param    docHandle
    *           The handle reference.
    *
    * @return   {@link Document} or <code>null</code> if handle invalid or doesn't contains
    *           Document object.
    */
   private static Document getDocument(handle docHandle) {
      if (!docHandle._isValid())
      {
         return null;
      }
      Object resource = docHandle.get();

      XDocumentImpl xDocument;

      if (resource instanceof XDocumentImpl)
      {
         xDocument = (XDocumentImpl) resource;
      }
      else
      {
         return null;
      }

      return (Document) xDocument.getNode();
   }

   /**
    * Constructs a new instance of the Progress X-Node-Reference object and
    * retrieves the handle variable associated with it. The approach is
    * slightly different. Each node will have additional parameter - Name
    * Space and node will be addressed by namespace and name pair instead
    * of the simple name.
    *
    * @param    h
    *           The handle where the new resource will be set as the referent.
    * @param    docHandle
    *           handle that should contains the X-Document object to create X-Node under.
    * @param    nsUri
    *           Name space associated with the current set of names.
    * @param    name
    *           The name of the node to create.
    * @param    type
    *           The type of the node to create.
    */
   public static void createXNodeReference(handle    h,
                                           handle    docHandle,
                                           character nsUri,
                                           character name,
                                           character type)
   {
      if (docHandle.isUnknown() || !docHandle._isValid())
      {
         return;
      }

      Document doc = getDocument(docHandle);
      if (doc == null)
      {
         return;
      }
      String nameImpl = null;
      String nsUriImpl = null;
      
      if (!nsUri.isUnknown())
      {
         nsUriImpl = nsUri.toStringMessage();
      }
      if (!name.isUnknown())
      {
         nameImpl = name.toStringMessage();
      }
      Integer nodeType = type.isUnknown() ?
              ELEMENT_NODE :
              convertAblNodeToNodeId(type.toStringMessage());

      final Node node;

      if (nodeType == ELEMENT_NODE)
      {
         node = doc.createElementNS(nsUriImpl, nameImpl);
      }
      else if (nodeType == ATTRIBUTE_NODE)
      {
         node = doc.createAttributeNS(nsUriImpl, nameImpl);
      }
      else
      {
         throw new DOMException(DOMException.TYPE_MISMATCH_ERR, "Unknown type");
      }

      h.assign(new XNodeRefImpl(node));
   }

   /**
    * Constructs a new instance of the Progress Sax-Attributes object and retrieves the handle
    * variable associated with it.
    * 
    * @param    h
    *           The handle where the new resource will be set as the referent.
    */
   public static void createSaxAttributes(handle h)
   {
      createSaxAttributes(h, new character());
   }

   /**
    * Constructs a new instance of the Progress Sax-Attributes object and retrieves the handle
    * variable associated with it.
    *
    * @param    h
    *           The handle where the new resource will be set as the referent.
    * @param    widgetPool
    *           The widget pool in which the object handle is created.
    */
   public static void createSaxAttributes(handle h, String widgetPool)
   {
      createSaxAttributes(h, new character(widgetPool));
   }

   /**
    * Constructs a new instance of the Progress Sax-Attributes object and retrieves the handle
    * variable associated with it.
    *
    * @param    h
    *           The handle where the new resource will be set as the referent.
    * @param    widgetPool
    *           The widget pool in which the object handle is created.
    */
   public static void createSaxAttributes(handle h, character widgetPool)
   {
      // TODO: implement this
   }
   
   /**
    * Constructs a new instance of the Progress Sax-Reader object and retrieves the handle
    * variable associated with it.
    * 
    * @param    h
    *           The handle where the new resource will be set as the referent.
    */
   public static void createSaxReader(handle h)
   {
      createSaxReader(h, new character());
   }

   /**
    * Constructs a new instance of the Progress Sax-Reader object and retrieves the handle
    * variable associated with it.
    *
    * @param    h
    *           The handle where the new resource will be set as the referent.
    * @param    widgetPool
    *           The widget pool in which the object handle is created.
    */
   public static void createSaxReader(handle h, String widgetPool)
   {
      createSaxReader(h, new character(widgetPool));
   }

   /**
    * Constructs a new instance of the Progress Sax-Reader object and retrieves the handle
    * variable associated with it.
    *
    * @param    h
    *           The handle where the new resource will be set as the referent.
    * @param    widgetPool
    *           The widget pool in which the object handle is created.
    */
   public static void createSaxReader(handle h, character widgetPool)
   {
      // TODO: implement this
   }
   
   /**
    * Constructs a new instance of the Progress Sax-Writer object and retrieves the handle
    * variable associated with it.
    * 
    * @param    h
    *           The handle where the new resource will be set as the referent.
    */
   public static void createSaxWriter(handle h)
   {
      createSaxWriter(h, new character());
   }

   /**
    * Constructs a new instance of the Progress Sax-Writer object and retrieves the handle
    * variable associated with it.
    *
    * @param    h
    *           The handle where the new resource will be set as the referent.
    * @param    widgetPool
    *           The widget pool in which the object handle is created.
    */
   public static void createSaxWriter(handle h, String widgetPool)
   {
      createSaxWriter(h, new character(widgetPool));
   }

   /**
    * Constructs a new instance of the Progress Sax-Writer object and retrieves the handle
    * variable associated with it.
    *
    * @param    h
    *           The handle where the new resource will be set as the referent.
    * @param    widgetPool
    *           The widget pool in which the object handle is created.
    */
   public static void createSaxWriter(handle h, character widgetPool)
   {
      // TODO: implement this
   }

   /**
    * Resolve the given numeric token type to a character ABL token type.
    *
    * @param   nodeId
    *          integer constant representing node id.
    *
    * @return  character token type or <code>unknow</code> if node id could not be resolved.
    */
   public static character convertNodeToAblNode(int nodeId)
   {
      return new character(nodeIdToNodeName.get(nodeId));
   }

   /**
    * Resolve the given numeric token type to a token name. The lookup is case-insensitive.
    *
    * @param   nodeName
    *          String constant representing token name.
    *
    * @return  int token type or <code>null</code> if name could not
    *          be resolved.
    */
   public static Integer convertAblNodeToNodeId(String nodeName)
   {
      if (nodeName == null)
      {
         return null;
      }
      return nodeNameToNodeId.get(nodeName.toUpperCase());
   }
}
